#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <gmerlin/avdec.h>
#include <gavl/gavl.h>

// gcc -O2 -o gavbench gavbench.c -lgmerlin_avdec `pkg-config --cflags --libs gavl`

int main (int argc, char **argv) {
  bgav_t *bgav;
  int stream;
  int track;
  char *file_name = argv[1];

  bgav = bgav_create();
  bgav_options_t *opt = bgav_get_options(bgav);
  bgav_options_set_sample_accurate(opt, 1);

  if (!bgav_open(bgav, file_name)) {
    fprintf( stderr, "Cannot open file %s\n", file_name);
    return (-1);
  }
  if (!bgav_can_seek_sample(bgav)) {
    fprintf(stderr, "%s supports no sample accuracy\n", file_name);
  }

  track = 0;
  stream = 0;
//bgav_num_audio_streams(bgav, track);
//bgav_num_video_streams(bgav, track);
  bgav_select_track(bgav, track);
  bgav_set_video_stream(bgav, stream, BGAV_STREAM_DECODE);
  bgav_start(bgav); // start decoder

  const gavl_video_format_t *vformat = bgav_get_video_format(bgav, stream);
//const gavl_audio_format_t *aformat = bgav_get_audio_format(bgav, stream);
//const bgav_metadata_t    *metadata = bgav_get_metadata(bgav, track);

  int frames       = gavl_time_scale(vformat->timescale, bgav_get_duration(bgav, track));
  int out_width    = vformat->frame_width;
  int out_height   = vformat->frame_height;
#if 0
  int duration     = gavl_time_to_seconds(bgav_get_duration(bgav, track));
  int framerate    = vformat->timescale;
  long int starttime = bgav_video_start_time(bgav, stream);
  long unsigned int s =  (out_width * out_height * gavl_pixelformat_bytes_per_pixel(vformat->pixelformat));

  char tp[GAVL_TIME_STRING_LEN];
  gavl_time_prettyprint(gavl_frames_to_time(framerate, 1, frames), tp);
  fprintf(stdout, "f=%ix%i\n", out_width, out_height); 
  fprintf(stdout, "s=%lu\n", s); 
  fprintf(stdout, "d=%s\n", tp); 
  fprintf(stdout, "d=%i s\n", duration);
  fprintf(stdout, "d=%i frames\n", frames);
  fprintf(stdout, "fr=%i fps\n", framerate);
  fprintf(stdout, "off=%li frames\n", starttime);
  fprintf(stderr, "Info: format=%s\n", gavl_pixelformat_to_string(vformat->pixelformat));
#endif

  int frameno=0;

  while (frameno < frames) { 
    bgav_seek_video(bgav, stream, frameno);
    gavl_video_frame_t *frame = gavl_video_frame_create(vformat); // TODO take this out of the loop ?!
    int result = bgav_read_video(bgav, frame, stream);

#if 0  
  long unsigned int s =  (out_width * out_height * gavl_pixelformat_bytes_per_pixel(vformat->pixelformat));
  fprintf(stderr, "Decoded %d frame(s), pts: %" PRId64 " (%lu bytes)\n", result, frame->timestamp,s);
#endif

#if 0 // convert
  int   render_fmt = GAVL_YUV_420_P;
//int   render_fmt = GAVL_RGB_24;

  gavl_video_format_t cur_format;
  gavl_video_format_copy(&cur_format, vformat);

  if (vformat->pixelformat != render_fmt){
    gavl_video_converter_t * cnv;

    gavl_video_format_t format_1;
    cnv = gavl_video_converter_create();
    gavl_video_format_copy(&format_1, &cur_format);
    format_1.pixelformat = render_fmt;

    gavl_video_frame_t * frame_1 = (gavl_video_frame_t*)0;
    frame_1 = gavl_video_frame_create(&format_1);

    gavl_video_options_t * opt;
    opt = gavl_video_converter_get_options(cnv);
    gavl_video_options_set_alpha_mode(opt, GAVL_ALPHA_BLEND_COLOR);    
    gavl_video_options_set_quality(opt, 2);    
    gavl_video_converter_init(cnv, vformat, &format_1);
    gavl_video_convert(cnv, frame, frame_1);
    gavl_video_converter_destroy(cnv);

    //assert (vformat->frame_height == format_1.frame_height);

    // swap frame and frame_1
    gavl_video_frame_destroy(frame);
    frame = frame_1;
    gavl_video_format_copy(&cur_format, &format_1);
  }
#endif

#if 0 // scale
  if (out_width != vformat->image_width || out_height != vformat->image_height) {
    gavl_video_scaler_t * scaler; // TODO: statically allocate this once
    gavl_video_format_t format_2;
    gavl_video_options_t * opt;
    gavl_rectangle_f_t src_rect;
    gavl_rectangle_i_t dst_rect;

    src_rect.w = vformat->image_width;
    src_rect.h = vformat->image_height;
    src_rect.x = 0;
    src_rect.y = 0;

    dst_rect.w = out_width;
    dst_rect.h = out_height;
    dst_rect.x = 0;
    dst_rect.y = 0;
  //printf("scaling: %fx%f -> %ix%i\n", src_rect.w, src_rect.h, dst_rect.w , dst_rect.h);

    gavl_video_frame_t * frame_2 = (gavl_video_frame_t*)0;

    gavl_video_format_copy(&format_2, &cur_format);
    format_2.image_width  = dst_rect.w + dst_rect.x;
    format_2.frame_width  = dst_rect.w + dst_rect.x;
    format_2.image_height = dst_rect.h + dst_rect.y;
    format_2.frame_height = dst_rect.h + dst_rect.y;

    frame_2 = gavl_video_frame_create(&format_2);

    scaler = gavl_video_scaler_create();
    opt = gavl_video_scaler_get_options(scaler);

    //gavl_video_options_set_quality(opt, 2);
    //gavl_video_options_set_scale_order(opt, 5); // Set the scale order for GAVL_SCALE_SINC_LANCZOS
    gavl_video_options_set_scale_mode(opt, GAVL_SCALE_QUADRATIC);
    //gavl_video_options_set_accel_flags(opt, GAVL_ACCEL_MMX);
 
    gavl_video_options_set_rectangles(gavl_video_scaler_get_options(scaler), &src_rect, &dst_rect);
    //printf("DEBUG scale: %s -> %s\n",gavl_pixelformat_to_string(cur_format.pixelformat), gavl_pixelformat_to_string(format_2.pixelformat));
 
    if (!gavl_video_scaler_init(scaler, &cur_format, &format_2)) {
      fprintf(stderr, "No scaling routine defined\n");
      return;
    }

    gavl_video_frame_clear(frame_2, &format_2);
    gavl_video_scaler_scale(scaler, frame, frame_2);

    // swap frame and frame_1
    gavl_video_frame_destroy(frame);
    frame = frame_2;
  }
#endif
    gavl_video_frame_destroy(frame);
    frameno++;
  }

  bgav_close(bgav); bgav=NULL;
  return 0;
}
